""" main app for a multi-threaded network port scanner """
#########
#  skan.py
#
#  Description:
#  multi-threaded network port scanner
#
#  Author:
#  Stephen Eaton <seaton@strobotics.com.au>

import argparse
import socket
import ipaddress
import threading
from queue import Queue

VERSION = "0.0.2"

# get and parse command line arguments
PARSER = argparse.ArgumentParser(
    description="Perform a network port scan \
                    of a host IP address"
)
PARSER.add_argument(
    "target", type=str, metavar="target_ip", help="Target IP address (xxx.xxx.xxx.xxx)"
)
PARSER.add_argument(
    "-s",
    "--startport",
    type=int,
    metavar="",
    default=1,
    help="Port number to start scanning (default: 1)",
    required=False,
)
PARSER.add_argument(
    "-e",
    "--endport",
    type=int,
    metavar="",
    default=1024,
    help="End port number of scan range (default: 1024,  max: 65535)",
    required=False,
)
PARSER.add_argument(
    "-t",
    "--threads",
    type=int,
    metavar="",
    default=32,
    help="number of scan threads to run (default: 32, max: 1024)",
    required=False,
)
PARSER.add_argument(
    "--version",
    action="version",
    version="%(prog)s v" + VERSION,
    help="prints version information",
)
ARGS = PARSER.parse_args()

# queue for all the ports to scan
SCAN_QUEUE = Queue()

# define a resource lock i.e. only one thread can print to stdio at a time
#   so thread needs to lock the resource while printing
MUTEX_PRINT = threading.Lock()


def validate_args():
    "arguments bounds checking and validation"

    # valide the IP address and exit with an error
    try:
        ipaddress.IPv4Address(ARGS.target)
    except ValueError:
        print("ERROR - IP address is invalid: %s" % ARGS.target)
        exit(1)

    if ARGS.startport <= 0:
        ARGS.startport = 1

    ARGS.endport = min(ARGS.endport, 65535)

    # check that start and end are correct
    ARGS.startport = min(ARGS.startport, ARGS.endport)
    # dont create unused threads if we can help it
    ARGS.threads = min(ARGS.threads, ARGS.endport - ARGS.startport)
    ARGS.threads = max(ARGS.threads, 1)
    ARGS.threads = min(ARGS.threads, 1024)


def scan_port(port):
    "scan_port will attempt to connect to the target IP on the given port"
    "if the connection is successful then prints the open port number"
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        conn = sock.connect((ARGS.target, port))

        # acquire lock and print to stdio
        with MUTEX_PRINT:
            print("port", port, ":open")

        conn.close()
    except:
        pass  # just ignore any exception of a non connection / timeout


def get_scan_job():
    "get the next port to scan from the queue"
    while True:
        # gets next port to scan from queue
        port = SCAN_QUEUE.get()
        scan_port(port)
        # signal queue that job has been completed
        SCAN_QUEUE.task_done()


def start_worker_threads(count):
    "starts the requested amount of worker threads"
    for _ in range(count + 1):
        w = threading.Thread(target=get_scan_job)
        w.daemon = True
        w.start()


def queue_ports():
    "queues the ports for scanning, each thread will de-queue a port to scan until queue is empty"
    for port in range(ARGS.startport, ARGS.endport + 1):
        SCAN_QUEUE.put(port)


def main():
    """Main application"""

    validate_args()

    print("")
    print(PARSER.prog, "v", VERSION)
    print(
        "Scanning IP:",
        ARGS.target,
        "ports:",
        ARGS.startport,
        "-",
        ARGS.endport,
        "threads:",
        ARGS.threads,
    )
    print("-=================================================================-")
    # setup ports to scan and start worker threads
    queue_ports()
    start_worker_threads(ARGS.threads)

    # wait for all scans to complete before exiting app
    SCAN_QUEUE.join()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['skanr']

package_data = \
{'': ['*']}

install_requires = \
['argparse>=1.4.0,<2.0.0', 'ipaddress>=1.0.23,<2.0.0']

entry_points = \
{'console_scripts': ['skanr = skanr.cli:main']}

setup_kwargs = {
    'name': 'skanr',
    'version': '0.0.2',
    'description': 'Multithreaded IPV4 port scanner written in python',
    'long_description': '# skanr\n\nMultithreaded network port scanner for IPV4 addresses\n\n## requirements\n\n1. known Target host IP Address\n\n## installation\n\n`pip install skanr`\n\n## usage\n\n### Help\n\nDisplay the help\n\n```\nskanr -h\nusage: skanr [-h] [-s] [-e] [-t] [--version] target_ip\n\nPerform a network port scan of a host IP address\n\npositional arguments:\n  target_ip          Target IP address (xxx.xxx.xxx.xxx)\n\noptional arguments:\n  -h, --help         show this help message and exit\n  -s , --startport   Port number to start scanning (default: 1)\n  -e , --endport     End port number of scan range (default: 1024, max: 65535)\n  -t , --threads     number of scan threads to run (default: 32, max: 1024)\n  --version          prints version information\n```\n\n### basic IP scan\n\nwill scan an IP address from ports 1-1024 using 32 threads\n\n```\nskanr 10.37.129.9\n\nskanr v 0.0.1\nScanning IP: 10.37.129.9 ports: 1 - 1024 threads: 32\n-=================================================================-\nport 21 :open\nport 23 :open\nport 22 :open\nport 53 :open\nport 80 :open\nport 110 :open\nport 111 :open\nport 139 :open\nport 143 :open\nport 445 :open\nport 901 :open\n```\n\n### scan port range on target\n\nTo scan a port range on the target machine use `-s` and `-e` to specify the start and end ports\n\nif no start ot end prot is given then defaults apply\n\n```\nskanr 10.37.129.9 -s 20 -e 100\n\nskanr v 0.0.1\nScanning IP: 10.37.129.9 ports: 20 - 100 threads: 32\n-=================================================================-\nport 21 :open\nport 22 :open\nport 23 :open\nport 53 :open\nport 80 :open\n```\n\n### specifing threads\n\nTo speed up the port scanning the number of threads can be passed using the `-t` argument up to a max of 1024 threads, to prevent starting unwanted threads, the number of threads started will never be greater than the number of ports to be scanned. \n\n```\nskanr 10.37.129.9 -e 65535 -t 1024\n\nskanr v 0.0.1\nScanning IP: 10.37.129.9 ports: 1 - 65535 threads: 1024\n-=================================================================-\nport 21 :open\nport 22 :open\nport 23 :open\nport 53 :open\nport 80 :open\nport 110 :open\nport 111 :open\nport 139 :open\nport 143 :open\nport 445 :open\nport 901 :open\nport 2049 :open\nport 6665 :open\nport 6666 :open\nport 6669 :open\nport 6667 :open\nport 6668 :open\nport 8787 :open\nport 37159 :open\nport 44180 :open\nport 50166 :open\n```\n',
    'author': 'Stephen Eaton',
    'author_email': 'seaton@strobotics.com.au',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/madeinoz67/skanr',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)
